import pydeck as pdk
import streamlit as st
import numpy as np
import xarray as xr
import altair as alt
from enum import IntEnum

INTRO = """\
    ---  
    MARSA
    DFG Transfer Project: Mitigation Assessments for Rice produced in Southeast Asia:
    Digital Platform for Quantification and Planning exemplified for Vietnam.

    Project summary:
    In contrast to the currently used Tier 2 approach, the IPCC Tier 3 approach is based on simulation models that could substantially increase the accuracy of the GHG estimates as long as the model is backed up by more detailed data on crop management. While the model LandscapeDNDC was validated for rice production in a previous project with the acronym ICON, the use of this model requires highly resolved databases as well as trained personnel proficient in pre- and post-processing of data inputs and outputs. Therefore, the project develops a user-friendly digital platform (see figure above) comprising a structured data repository and a Graphics User Interface (GUI) that simplifies the model runs and allows high flexibility in terms of the selection of different databases as inputs as well as output formats. The Vietnamese project partner, the Institute of Agricultural Environment, has been assigned to compute the emissions from rice production in the forthcoming 4th National Communications so that the project accomplishments can swiftly be translated into a GHG inventory at the national scale based on Tier 3 which would be the 1st of its kind for any rice growing country.
    
    ---  
"""

class dotdict(dict):
    """dot.notation access to dictionary attributes"""
    __getattr__ = dict.get
    __setattr__ = dict.__setitem__
    __delattr__ = dict.__delitem__

color = dotdict(green = '#bda346',
                brown = '#77ae79',
                gray = '#f0f2f6')

class GWP(IntEnum):
    N2O = 298
    CH4 = 34

def convert_ch4_gwp(value):
    return value * (16./12.) * GWP.CH4

def convert_n2o_gwp(value):
    return value * (44./28.) * GWP.N2O


# dicts for nicer naming
irri_names = {'awd': 'Alternate Wet-Dry (AWD)',
              'midseason-drainage': 'Midseason Drainage (MD)',
              'continuously-flooded': 'Continuously Flooded (CF)'}
              
residues_names = {'global-crop-residues': 'Global',
                  'iae': 'IAE',
                  'ipcc': 'IPCC'}

names = {'ch4_gwp': 'CH₄ Emission',
         'n2o_gwp': 'N₂O Emission', 
         'total_gwp': 'Total Emission'}

def scale(x, xmin, xmax):
    return (x - xmin)/(xmax-xmin)

def compute_rgb(data, var):
    """a wacky way to compute three different color ranges"""
    bcol = {'ch4_gwp': 20, 'n2o_gwp': 60}.get(var, 100)
    return [[(255-bcol*2), 150 + 100*(1-d), bcol*2.5] for d in data]

def generate_json(ds, var='ch4_gwp', bounds=None):
    v = ds[var] #.mean(dim='year')
        
    df = v.to_dataframe().reset_index()
    df = df[df[var] > 0]
    x = df[var]
    values = scale(x, *bounds) if bounds else x
    locs = list(zip(df.lon, df.lat))
    colors = compute_rgb(values, var)
    return [{'h': 60*h, 'pos': list(p), 'c': c} for h,p,c in zip(values,locs,colors)]

def process_df(df, vars):
    df_ = df.reset_index()[vars] if vars else df.reset_index()
    if vars is None:
        vars = list(df.columns.values)

    # rename columns
    nice_names = {'n2o_gwp': 'N₂O', 'ch4_gwp': 'CH₄'}
    for v in vars:
        df_ = df_.rename({v: nice_names.get(v)}, axis=1)

    df_ = df_.melt('year', var_name='variable', value_name='value')
    return (df_.year, df_)

def bar_plot(ds, vars=None):
    
    df = ds.to_dataframe() * 1e-6
    index_, df_ = process_df(df, vars)
    units_ = 'Gg CO₂-eq yr⁻¹'
    
    c = alt.Chart(df_).mark_bar().encode(
            alt.X('year:O', title=''),
            alt.Y('value:Q', title=f'{units_}', 
                   scale=alt.Scale(domain=[0, 120_000])), 
            alt.Color('variable:N', 
                      scale=alt.Scale(range=[color.green, color.brown]),
                      legend=alt.Legend(title='')),
            order=alt.Order('value', sort='descending')
        ).configure_axis(
            grid=False
        ).properties(
            height=200, 
            background=color.gray
        )
    st.sidebar.altair_chart(c, use_container_width=True)  


@st.cache_resource()
def load_data( name):
    return xr.open_dataset( f'./data/VN_arable-{name}-yearly_inventory.nc')

@st.cache_resource()
def load_data_admin():
    return xr.open_dataset('./data/admin.nc')

def main():
    st.title("Greenhouse gas emissions from rice production in Vietnam")
    st.markdown("""\
        [LandscapeDNDC](https://ldndc.imk-ifu.kit.edu) simulations conducted
        as part of the [MARSA [1]](https://bgc.imk-ifu.kit.edu/marsa)
        project realized at [KIT Campus Alpin](https://www.imk-ifu.kit.edu).
        """)

    intro = st.checkbox("[1] show some background information", value=False)
    if intro:
        st.write(INTRO)

    st.sidebar.subheader("Data selection")

    irri = st.sidebar.radio("Irrigation", ["continuously-flooded", "midseason-drainage", "awd"], 0, format_func=irri_names.get)
    residues = st.sidebar.radio("Residues", ["global-crop-residues", "iae", "ipcc"], 0, format_func=residues_names.get)
    ghg = st.sidebar.radio("Emissions", ["n2o_gwp", "ch4_gwp", "total_gwp"], 2, format_func=names.get)

    region_ids = []
    province_ids = []
    resolution = st.sidebar.radio("Country", ["Country", "Regions", "Provinces"], 0)
    if resolution == "Country":
        pass
    if resolution == "Regions":
        region = st.sidebar.radio("Region", ["Northeast", "Northwest", "Red River Delta", "Central", "Southeast", "Mekong River Delta"], 0)
        region_ids = dict({"Northeast": [2], "Northwest": [7], "Red River Delta":[4], "Central":[5,6,8], "Southeast":[3], "Mekong River Delta":[1]})[region]
    elif resolution == "Provinces":
        province = st.sidebar.radio("Province", ["BacKan", "CaoBang", "HaGiang", "LaiChau", "LangSon", "LaoCai", "PhuTho", "QuangNinh", "SonLa",
                                                 "TuyenQuang", "YenBai", "BacGiang", "HaNoi", "HaiDuong", "HaiPhong", "HungYen", "NamDinh", "NinhBinh",
                                                 "ThaiNguyen", "BacNinh", "HaNam", "HaiDuong", "HungYen", "NamDinh", "NinhBinh", "DienBien", "HoaBinh",
                                                 "SonLa", "BinhDinh", "DaNang", "KhanhHoa", "Phuyen", "QuangNam", "QuangNgai",
                                                 "ThuaThienHue", "DakLak", "GiaLai", "KonTum", "LamDong", "BacLieu", "BenTre", "BinhDuong", "BinhPhuoc",
                                                 "CaMau", "CanTho", "DongNai", "HauGiang", "KienGiang", "LongAn", "SocTrang", "TayNinh", "VungTau",
                                                 "AnGiang", "DongThap", "TienGiang", "TraVinh", "VinhLong"])

        province_ids = dict({"DongThap": [1], "SocTrang": [2], "TienGiang": [3], "TraVinh": [4], "VinhLong": [5], "AnGiang": [6], "BacLieu": [7], "BenTre": [8],
                             "CaMau": [9], "CanTho": [10], "HauGiang": [11], "KienGiang": [12], "LongAn": [13], "BacGiang": [14], "BacKan": [15], "CaoBang": [16],
                             "HaGiang": [17], "LaoCai": [18], "LangSon": [19], "PhuTho": [20], "QuangNinh": [21], "ThaiNguyen": [22],
                             "TuyenQuang": [23], "YenBai": [24], "DongNai": [25], "VungTau": [26], "BinhDuong": [27], "BinhPhuoc": [28], "BinhThuan": [29],
                             "HoChiMinhCity": [30], "NinhThuan": [31], "TayNinh": [32], "BacNinh": [33], "HaNoi": [34], "HaNam": [35], "HaTay": [36], "HungYen": [37],
                             "HaiDuong": [38], "HaiPhong": [39], "NamDinh": [40], "NinhBinh": [41], "Thaibinh": [42], "VinhPhuc": [43], "HaTinh": [44], "NgheAn": [45],
                             "QuangBinh": [46], "QuangTri": [47], "ThuaThienHue": [48], "ThanhHoa": [49], "DaNang": [50], "BinhDinh": [51], "KhanhHoa": [52],
                             "Phuyen": [53], "QuangNam": [54], "QuangNgai": [55], "DienBien": [56], "HoaBinh": [57], "LaiChau": [58], "SonLa": [59], "DacNong": [60],
                             "DakLak": [61], "GiaLai": [62], "KonTum": [63], "LamDong": [64]})[province]

    # get netcdf data
    ds_admin = load_data_admin()
    #rice_fr [%]
    area = ds_admin.area_ha * 10000 * ds_admin.agrifrac * 0.01 * ds_admin.rice_fr * 0.01

    #VN_arable-{name}-yearly_inventory.nc
    ds = load_data( f"{irri}-{residues}")

    if len(region_ids) > 0:
        ds = ds.where( ds_admin['regionid'].astype(int).isin(region_ids))
    elif len(province_ids) > 0:
        ds = ds.where( ds_admin['provinceid'].astype(int).isin(province_ids))


    ds['n2o_gwp'] = convert_n2o_gwp( ds.dN_n2o_emis )
    ds['ch4_gwp'] = convert_ch4_gwp( ds.dC_ch4_emis )
    ds['total_gwp'] = ds.ch4_gwp + ds.n2o_gwp

    # get min/ max values
    min_val, max_val = ds.total_gwp.min().values, ds.total_gwp.max().values

    # calculate total emissions (scaled by area)
    ds_sum = (ds[['ch4_gwp','n2o_gwp']] * area).sum(dim=["lat", "lon"])

    year = st.slider("Select simulation year", 2014, 2012, 2015)

    COUNTRIES = "https://d2ad6b4ur7yvpq.cloudfront.net/naturalearth-3.3.0/ne_50m_admin_0_scale_rank.geojson"
    COUNTRIES = "./ne_50m_admin_0_scale_rank.geojson"
    ll = pdk.Layer(
        "GeoJsonLayer",
        id="base-map",
        data=COUNTRIES,
        stroked=False,
        filled=True,
        get_fill_color=[100, 200, 200],
    )

    layer = pdk.Layer(
        'GridCellLayer',
        generate_json(ds.sel(year=year) , var=ghg, bounds=(min_val, max_val)),
        get_position='pos',
        extruded= True, 
        get_fill_color='c',
        get_elevation = 'h',
        cell_size=10000,
        elevation_scale= 1000,
        coverage=.85,
        method='nearest'
    )

    view_state = pdk.ViewState(latitude=15, longitude=105, zoom=4.5, pitch=33)

    r = pdk.Deck(layers=[layer],
                 initial_view_state=view_state,
                 map_style='mapbox://styles/mapbox/light-v10',
                 #map_style='mapbox://styles/mapbox/cjaudgl840gn32rnrepcb9b9g',
                # CURRENTLY NOT WORKING IN 0.53 
                #  tooltip={
                #     'html': '<b>Emission:</b> {h}',
                #     'style': {
                #         'color': 'white'
                #         }
                #     }
                )
    st.write(r)

    def compute_stats(ds, how='annual'):
        # stats - selected year
        ds_ = ds.sel(year=year) if how == 'annual' else ds_sum.mean(dim='year')
        n2o = ds_.n2o_gwp.values * 1e-6
        ch4 = ds_.ch4_gwp.values * 1e-6
        tot = n2o + ch4
        return (tot, n2o, ch4)

    # stats - selected year and average
    tot_sel, n2o_sel, ch4_sel = compute_stats(ds_sum, how="annual")
    tot_avg, n2o_avg, ch4_avg = compute_stats(ds_sum, how="average")

    st.sidebar.subheader("Summary stats")

    #bar_plot(ds_sum)

    st.sidebar.info(f"""\
        ### Total emissions

        |     |   {year}        |  avg             |   
        |-----|----------------:|-----------------:|  
        |CH₄  |`{ch4_sel:,.0f}` | `{ch4_avg:,.0f}` |
        |N₂O  |`{n2o_sel:,.0f}` | `{n2o_avg:,.0f}` |
        |sum  |`{tot_sel:,.0f}` | `{tot_avg:,.0f}` |

        units: Gg CO₂-eq yr⁻¹      
        """)


if __name__ == "__main__":
    main()
