import pydeck as pdk
import pandas as pd
import streamlit as st
import numpy as np
import xarray as xr
import altair as alt
from enum import IntEnum
import os
import matplotlib.pyplot as plt

INTRO = """\
    ---  
    MARSA
    DFG Transfer Project: Mitigation Assessments for Rice produced in Southeast Asia:
    Digital Platform for Quantification and Planning exemplified for Vietnam.

    Project summary:
    In contrast to the currently used Tier 2 approach, the IPCC Tier 3 approach is based on simulation models that could substantially increase the accuracy of the GHG estimates as long as the model is backed up by more detailed data on crop management. While the model LandscapeDNDC was validated for rice production in a previous project with the acronym ICON, the use of this model requires highly resolved databases as well as trained personnel proficient in pre- and post-processing of data inputs and outputs. Therefore, the project develops a user-friendly digital platform (see figure above) comprising a structured data repository and a Graphics User Interface (GUI) that simplifies the model runs and allows high flexibility in terms of the selection of different databases as inputs as well as output formats. The Vietnamese project partner, the Institute of Agricultural Environment, has been assigned to compute the emissions from rice production in the forthcoming 4th National Communications so that the project accomplishments can swiftly be translated into a GHG inventory at the national scale based on Tier 3 which would be the 1st of its kind for any rice growing country.
    
    ---  
"""


def list_directories_with_files( base_directory, file_ending):
    """
    Lists all directories inside the base directory that contain at least one file ending with the specified file_ending.

    :param base_directory: The directory to search within.
    :param file_ending: The file ending to look for.
    :return: A list of directory paths.
    """
    matching_directories = []

    # Walk through the base directory
    for root, dirs, files in os.walk(base_directory):
        # Check if any file ends with the specified ending
        if any(file.endswith(file_ending) for file in files):
            matching_directories.append(root)

    return matching_directories

def get_file( base_directory, file_ending):
    for root, dirs, files in os.walk( base_directory):
        # Check if any file ends with the specified ending
        for file in files:
            if file.endswith( file_ending):
                return file
    return None
    
    
class dotdict(dict):
    """dot.notation access to dictionary attributes"""
    __getattr__ = dict.get
    __setattr__ = dict.__setitem__
    __delattr__ = dict.__delitem__

color = dotdict(green = '#bda346',
                brown = '#77ae79',
                gray = '#f0f2f6')

class GWP(IntEnum):
    N2O = 298
    CH4 = 34

def convert_ch4_gwp(value):
    return value * (16./12.) * GWP.CH4

def convert_n2o_gwp(value):
    return value * (44./28.) * GWP.N2O


# dicts for nicer naming
irri_names = {'awd': 'Alternate Wet-Dry (AWD)',
              'midseason-drainage': 'Midseason Drainage (MD)',
              'continuously-flooded': 'Continuously Flooded (CF)'}

@st.cache_resource()
def load_data( name):
    return xr.open_dataset( f'./data/VN_arable-{name}-yearly_inventory.nc')


import argparse
# Instantiate the parser
parser = argparse.ArgumentParser(description='Optional app description')
parser.add_argument('--data', type=str,
                    help='...')
args = parser.parse_args()

def main():

    st.set_page_config(layout="wide")
    font = {'family' : 'sans-serif', 'size': 20}
    plt.rc('font', **font)

    #consider all directories having required output
    file_ending = 'soilchemistry-daily.txt'

    if args.data == 'local':
        base_dir = './data/sitescale/'
        simulation_dirs = list_directories_with_files( base_dir, file_ending)
    else:
        base_dir = '/opt/kit-model/model-outputs/rice/runs/'
        simulation_dirs = [base_dir+'rice_runid_'+str(i) for i in st.query_params["uids"].split(',')]

    simulation_names = [sim.split("/")[-1] for sim in simulation_dirs]

    #Define graphs
    graphs = [{ "value": "dC_ch4_emis[kgCha-1]", "label": "CH4 [kgCha-1]", "output":  "soilchemistry-daily" },
              { "value": "dN_n2o_emis[kgNha-1]", "label": "N2O [kgNha-1]", "output":  "soilchemistry-daily" },
              { "value": "DW_above[kgDWm-2]",      "label": "AGB [kgDWm-2]", "output":  "physiology-daily" }]
    st.sidebar.subheader("Add graphs")
    if ( st.sidebar.checkbox( f"Surfacewater", value=False)):
        graphs.append( {"value": "surfacewater[mm]", "label": "Surfacewater [mm]", "output": "watercycle-daily"})
    if ( st.sidebar.checkbox( f"Soilwater", value=False)):
        graphs.append( {"value": "soilwater_5cm[%]", "label": "Soilwater [mm]",    "output": "watercycle-daily"})
    nd_plots = len( graphs)

    plotheight = st.sidebar.slider("Scale graph", 0.0, 100.0, 35.0)

    st.sidebar.subheader("Simulation")
    sims = []
    for sim_name, sim_dir in zip( simulation_names, simulation_dirs):
        check_value = False
        if sims == []:
            check_value = True
        sims.append( {'check': st.sidebar.checkbox( f"{sim_name}", value=check_value),
                      'name': sim_name, 'dir': sim_dir})

    #Data for summary stats
    #Add dummy data to determine dtypes
    df_results = pd.DataFrame.from_dict({'ch4': [-99.99], 'n2o': [-99.99], 'yield': [-99.99], 'ch4_n2o':[-99.99], 'ch4_n2o_yield':[-99.99],
                                         'index':['None']})
    for sim in sims:
        df_data = pd.read_table( f"{sim['dir']}/{get_file( sim['dir'], 'dss-yearly.txt')}")
        df_data["datetime"] = pd.to_datetime( df_data["datetime"])
        df_data.set_index("datetime", inplace=True)
        df_data = df_data[["aC_ch4_emis[kgCha-1]", "aN_n2o_emis[kgNha-1]", "aDW_fru[kgDWha-1]"]]
        df_data = df_data.loc[(df_data.index >= pd.to_datetime("2015-01-01")),]

        df_add = pd.DataFrame.from_dict({'ch4': [df_data["aC_ch4_emis[kgCha-1]"].mean()],
                                         'n2o': [df_data["aN_n2o_emis[kgNha-1]"].mean()],
                                         'yield': [df_data["aDW_fru[kgDWha-1]"].mean()],
                                         'ch4_n2o':[-99.99], 'ch4_n2o_yield':[-99.99], 'index':[sim['name']]})
        df_results = pd.concat([df_results, df_add])

    df_results.set_index('index', inplace=True)
    df_results = df_results.rename( columns={"aC_ch4_emis[kgCha-1]": "ch4",
                                             "aN_n2o_emis[kgNha-1]": "n2o"})

    df_results["ch4_n2o"] = (df_results["ch4"] * GWP['CH4'] + df_results["ch4"] * GWP['N2O']) * 1.0e-3
    df_results["ch4_n2o_yield"] = df_results["ch4_n2o"] / df_results["yield"]
    df_results.drop('None', inplace = True)


    for gi, graph in enumerate( graphs):
        df_all = pd.DataFrame()
        for sim in sims:
            if sim['check']:
                f_out = get_file( sim['dir'], f'{graph["output"]}.txt')
                df_temp = pd.read_table( f"{sim['dir']}/{f_out}")
                df_temp["datetime"] = pd.to_datetime( df_temp["datetime"])
                df_temp.set_index("datetime", inplace=True)

                df_temp = df_temp[[graph["value"]]]
                df_temp = df_temp.rename( columns={graph["value"]: sim["name"]})

                df_all = pd.concat([df_all, df_temp], axis=1)
                date_range = pd.date_range(start=df_all.index.min(), end=df_all.index.max(), freq="D")
                df_all = df_all.reindex( date_range)

        df_all = df_all.loc[(df_all.index >= pd.to_datetime("2015-01-01")),]

        # Compute the day of the year
        df_all["day_of_year"] = df_all.index.dayofyear

        # Group by day of the year and calculate mean and standard deviation
        df_grouped = df_all.groupby("day_of_year")
        mean_values = df_grouped.mean()
        std_values = df_grouped.std()
        #nicer plot if biomass is set to zero
        if graph["value"] == "DW_above[kgDWm-2]":
            mean_values.fillna(0, inplace=True)
        graphs[gi].update({'data_mean': mean_values, 'data_std': std_values})


    have_graph = False
    for sim in sims:
        if sim['check']:
            have_graph = True

    if have_graph:
        fig, ax = plt.subplots(nd_plots, 1, figsize=(20, plotheight/10.0*nd_plots))
        for ax_index, graph in enumerate( graphs):

            sim_output = graph['output']
            sim_value = graph['value']
            sim_label = graph['label']
            mean_values = graph['data_mean']
            std_values = graph['data_std']

            for col in mean_values.columns:
                r1 = mean_values[col] - std_values[col]
                r2 = mean_values[col] + std_values[col]
                ax[ax_index].plot( mean_values.index, mean_values[col], label=col)
                ax[ax_index].fill_between(mean_values.index, r1.values, r2.values, alpha=0.2, label="")
            ax[ax_index].legend( fontsize=15, loc ="upper right")
            ax[ax_index].set_ylabel( sim_label)

        ax[nd_plots-1].set_xlabel("Day of year")
        plt.tight_layout()
        st.pyplot(fig)


        table_string = f"""### Summary statistics

| Simulation | CH₄ | N₂O | CH₄+N₂O (CO₂ eq.) | Yield | Yield scaled CH₄+N₂O (CO₂ eq.) |
|------------|----:|----:|------------------:|------:|-------------------------------:|
"""
        for sim in sims:
            if True: #sim['check']:
                table_string += f"|  {sim['name']} "
                table_string += f"| {df_results.loc[sim['name'], 'ch4']:,.0f} "
                table_string += f"| `{df_results.loc[sim['name'], 'n2o']:,.1f}` "
                table_string += f"| `{df_results.loc[sim['name'], 'ch4_n2o']:,.0f}` "
                table_string += f"| `{df_results.loc[sim['name'], 'ch4_n2o']:,.0f}` "
                table_string += f"""| `{df_results.loc[sim['name'], 'ch4_n2o']:,.0f}` |\n"""

        table_string += f"""
CH₄: [kg CH₄ ha⁻¹ yr⁻¹] \\
N₂O: [kg N₂O ha⁻¹ yr⁻¹] \\
CH₄+N₂O: [Mg CO₂ eq. ha⁻¹ yr⁻¹] \\
Yield: [t Dry Weight ha⁻¹ yr⁻¹] \\
Yield scaled CH₄+N₂O: [CO₂ eq. per Dry Weight] 
"""

        st.info(table_string)

if __name__ == "__main__":
    main()
